#include <formatio.h>
#include <userint.h>
#include <stdlib.h>
#include "mxOX8042.h"
#include "mxOX804u.h"

/*= Metrix OX8042 Oscilloscope Application Example Source File ==============*/
/* Original Release: February 1998                                           */
/* By: Olivier Lemaire, Metrix, France                                       */
/*     PH. +33(0)450642222  Fax  +33(0)450642200                             */
/*     e-mail: Metrix@dial.oleane.com                                        */
/*                                                                           */
/* Modification History: None                                                */
/*===========================================================================*/

/*****************************************************************************/
/*= GLOBAL VARIABLE DECLARATIONS ============================================*/
/*****************************************************************************/
static int panelHandle[10];
static unsigned int instrHandle;
static int      uir_code;
ViStatus        uir_err;
char        errMessage[256];

/*****************************************************************************/
/*= MESSAGE ARRAYS ==========================================================*/
/*****************************************************************************/

/*- Init Panel control helps ------------------------------------------------*/

static char *initCtrlsHelpStr[4] = {"\n\
This control selects serial port of the device that is to be\n\
initialized.\n\n\
Valid Range:  Serial port 1 to 8\n\n\
Default Value :  Serial port 1","\n\
This control specifies if an ID Query is sent to the\n\
instrument during the initialization procedure.\n\n\
Valid Range:\n\
VI_OFF (0) - Skip Query\n\
VI_ON  (1) - Do Query (Default Value)","\n\
This control specifies if the instrument is to be reset to its\n\
power-on settings during the initialization procedure.\n\n\
Valid Range:\n\
VI_OFF (0) - Don't Reset (Default Value)\n\
VI_ON  (1) - Reset Device"};    

/*- Acquire Waveform Panel control helps ------------------------------------*/     

static char *measCtrlsHelpStr[4] = {"\n\
This control sets the timebase of the oscilloscope.\n\n\
Valid Values:\n\
0 - 5 ns/div\n\
1 - 10 ns/div\n\
2 - 20 ns/div\n\
3 - 50 ns/div\n\
4 - 0.1 us/div\n\
5 - 0.2 us/div\n\
6 - 0.5 us/div\n\
7 - 1 us/div\n\
8 - 2 us/div\n\
9 - 5 us/div\n\
10 - 10 us/div\n\
11 - 20 us/div\n\
12 - 50 us/div  (Default Value)\n\
13 - 100 us/div\n\
14 - 200 us/div\n\
15 - 500 us/div\n\
16 - 1 ms/div\n\
17 - 2 ms/div\n\
18 - 5 ms/div\n\
19 - 10 ms/div\n\
20 - 20 ms/div\n\
21 - 50 ms/div\n\
22 - 100 ms/div\n\
23 - 200 ms/div","\n\
This control sets the input range of the selected\n\
channel in units of volts per division.\n\n\
Valid Values:\n\
0 - 10 mV/div\n\
1 - 20 mV/div\n\
2 - 50 mV/div\n\
3 - 100 mV/div\n\
4 - 200 mV/div\n\
5 - 500 mV/div\n\
6 - 1 V/div\n\
7 - 2 V/div   (Default Value)\n\
8 - 5 V/div\n\
9 - 10 V/div\n\
10 - 20 V/div\n\
11 - 50 V/div\n\
12 - 100 V/div\n\
13 - 200 V/div","\n\
This control selects the input coupling\n\
of the selected channel.\n\n\
Valid Values:\n\
0 - AC  (Default Value)\n\
1 - DC\n\
2 - GND","\n\
This control displays the waveform acquired\n\
by the instrument. Displayed waveform consists\n\
of 1000 samples acquired on channel 1 (trace S1)."};

/*- Background Panel Help ---------------------------------------------------*/

static char backgroundPnlHelpStr[] = {"\n\
This is a simple demo program using the Metrix OX8042/62\n\
Oscilloscope instrument driver provided in the package."};

/*- Init Panel Help ---------------------------------------------------------*/

static char initPnlHelpStr[] = {"\n\
This panel performs the following initialization actions:\n\n\
- Opens a session to the Default Resource Manager resource and a\n\
session to the specified device using the interface and address\n\
specified in the Resource_Name control.\n\n\
- Performs an identification query on the Instrument.\n\n\
- Resets the instrument to a known state.\n\n\
- Sends initialization commands to the instrument that set any\n\
necessary programmatic variables such as Headers Off, Short\n\
Command form, and Data Transfer Binary to the state necessary\n\
for the operation of the instrument driver.\n\n\
- Returns an Instrument Handle which is used to differentiate\n\
between different sessions of this instrument driver.\n\n\
- Each time this function is invoked a Unique Session is opened.\n\
It is possible to have more than one session open for the same\n\
resource."};

/*- Acquire Waveform Panel Help ---------------------------------------------*/

static char measPnlHelpStr[] = {"\n\
This panel acquires a single channel waveform from the channel 1\n\
of the instrument. Each time you press the button labelled\n\
\"New Measurement\", the instrument's timebase, input voltage\n\
range and input coupling are configured and new waveform is\n\
acquired and displayed on this panel."};

/*= Main Function ===========================================================*/

void main ()
{
    panelHandle[BCKGRND] = LoadPanel (0, "mxOX804u.uir", BCKGRND);
    panelHandle[MEAS] = LoadPanel (panelHandle[BCKGRND], "mxOX804u.uir", MEAS);
    panelHandle[INIT] = LoadPanel (panelHandle[BCKGRND], "mxOX804u.uir", INIT);
    DisplayPanel (panelHandle[BCKGRND]);
    DisplayPanel (panelHandle[INIT]);
    uir_code = RunUserInterface ();
}

/*===========================================================================*/
/* Function: Initialize Instrument                                           */
/* Purpose:  This is a callback function of the Continue button on the       */
/*           Initialize panel. It initializes the instrument and switches to */
/*           the panel Configure.                                            */
/*===========================================================================*/
int initInstrument (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    short addr, rst, id;
    char instrDescr[20];

    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (panelHandle[INIT], INIT_ADDRESS, &addr);
            GetCtrlVal (panelHandle[INIT], INIT_ID, &id);
            GetCtrlVal (panelHandle[INIT], INIT_RST, &rst);
            
            /*- Initializing the instrument ---------------------------------*/
            Fmt (instrDescr, "ASRL%d[b2]", addr);
            if (mxOX8042_init (instrDescr, id, rst, &instrHandle)) {
                MessagePopup ("Error", "Initialization error. Check your connections and make sure you have the right address.");
                return (0);            
            }
            else {
                HidePanel (panelHandle[INIT]);
                DisplayPanel (panelHandle[MEAS]);
            }
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Help","This button causes the instrument to be initialized.");
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Control Help                                                    */
/* Purpose:  This is a callback function of all controls that configure the  */
/*           instrument. On the right mouse-click on the control a help      */
/*           window describing its purpose is displayed.                     */
/*===========================================================================*/
int controlHelp (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_RIGHT_CLICK) {
        if (panel == panelHandle[INIT]) {
            MessagePopup ("Help", initCtrlsHelpStr[control-4]);
        }
        if (panel == panelHandle[MEAS]) {                                
            MessagePopup ("Help", measCtrlsHelpStr[control-4]);
        }
    }
    return 0;
}

/*===========================================================================*/
/* Function: New Measurement                                                 */
/* Purpose:  This is a callback function of the New Measurement button on    */
/*           the Acquire Waveform panel. New waveform is acquired by issuing */
/*           function Example of the Metrix OX8042 Oscilloscope instrument   */
/*           driver.                                                         */
/*===========================================================================*/
int newMeasurement (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    double data[1000], dt;
    int error;
    int timebase, vrange, coup, n;
    char errMsg[256];

    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (panelHandle[MEAS], MEAS_TIMEBASE, &timebase);
            GetCtrlVal (panelHandle[MEAS], MEAS_VOLTRANGE, &vrange);
            GetCtrlVal (panelHandle[MEAS], MEAS_COUPLING, &coup);
            
            /* Acquiring Waveform -------------------------------------------*/
            if (error = mxOX8042_example (instrHandle, timebase, vrange, coup, data, &dt, &n)) {
                mxOX8042_errorMessage (VI_NULL, error, errMsg);
                if (error >= 0xBFFF0000) {
                
                    /*- Critical error, no communication --------------------*/
                    Fmt (errMsg, "%s[a]<\nThis program will now terminate!");
                    MessagePopup ("Critical error", errMsg);
                    QuitUserInterface (uir_code);
                }       
                else {
                    
                    /*- Non-critical Error, can continue --------------------*/
                    mxOX8042_errorMessage (VI_NULL, error, errMsg);
                    MessagePopup ("Error", errMsg);
                    return 0;
                }   
            }
            DeleteGraphPlot (panelHandle[MEAS], MEAS_OUTPUT, -1, VAL_IMMEDIATE_DRAW);
            
            PlotWaveform (panelHandle[MEAS], MEAS_OUTPUT, data, n, VAL_DOUBLE, 1.0, 0.0, 0.0, 1000.0*dt,
                          VAL_THIN_LINE, VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_RED);
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Help","Press this button to configure the instrument and acquire new waveform.");
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Panel Help                                                      */
/* Purpose:  This is a callback function of the menu bar. It displays a help */
/*           window describing panel being used.                             */
/*===========================================================================*/
void panelHelp (int menuBar, int menuItem, void *callbackData, int panel)
{
    if (panel == panelHandle[BCKGRND])
        MessagePopup ("Help", backgroundPnlHelpStr);
    if (panel == panelHandle[INIT])
        MessagePopup ("Help", initPnlHelpStr);
    if (panel == panelHandle[MEAS])
        MessagePopup ("Help", measPnlHelpStr);
        
    return;
}

/*===========================================================================*/
/* Function: Launch Close Dialog                                             */
/* Purpose:  This is a callback function of the Quit buttons. It pops-up     */
/*           a dialog panel to choose whether to quit or not.                */
/*===========================================================================*/
int launchClose (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            if (ConfirmPopup ("Close", "Do you really want to quit?")) {
                if (panel != panelHandle[INIT]) {  
                    if ((uir_err = mxOX8042_close (instrHandle)) < 0) {
                        MessagePopup("Error","Application function failed when closing the instrument.");
                        mxOX8042_errorMessage(VI_NULL, uir_err, errMessage);
                        MessagePopup("Error",errMessage);
                    } 
                }   
                QuitUserInterface (uir_code);
            }   
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Help","This button will pop-up a panel to close the instrument.");
            break;
    }
    return 0;
}

/*= End =====================================================================*/
